# @summary Configures the servicenow
#
# @example
#  include servicenow_reporting_integration::incident_management
# @param [String[1]] instance
#   The FQDN of the ServiceNow instance
# @param [String[1]] caller_id
#   The sys_id of the incident's caller as specified in the sys_user table
#  parameter to the empty string ''.
# @param [Optional[String[1]]] user
#   The username of the account with permission to query data
# @param [Optional[Sensitive[String[1]]]] password
#   The password of the account used to query data from Servicenow
# @param [Optional[Sensitive[String[1]]]] oauth_token
#   An OAuth access token created in Servicenow that can be used in place of a
# @param [Optional[String[1]]] pe_console_url
#   The PE console url
#   username and password.
# @param [Optional[String[1]]] category
#   The incident's category
# @param [Optional[String[1]]] subcategory
#   The incident's subcategory
# @param [Optional[String[1]]] contact_type
#   The incident's contact type
# @param [Optional[Integer]] state
#   The incident's state
# @param [Optional[Integer]] impact
#   The incident's impact
# @param [Optional[Integer]] urgency
#   The incident's urgency
# @param [Optional[String[1]]] assignment_group
#   The sys_id of the incident's assignment group as specified in the
#   sys_user_group table
# @param [Optional[String[1]]] assigned_to
#   The sys_id of the user assigned to the incident as specified in the
#   sys_user table. Note that if assignment_group is also specified, then
#   this must correspond to a user who is a member of the assignment_group.
# @param [Optional[Array[String[1]]]] allow_list
#   An array of the name of environments that the user wants to allow data to be sent
# @param [Optional[Array[String[1]]]] block_list
#   An array of the name of environments that the user does not want data to be sent
# @param [Servicenow_reporting_integration::ReportCategories] incident_creation_conditions
#   The incident creation conditions. The report processor will create incidents for reports
#   that satisfy at least one of the specified conditions. For example, if you use the default
#   value (`['failures', 'corrective_changes']`), then the report processor will create an
#   incident if the report had any failures _or_ corrective changes.
#   Note: Set this parameter to `['never']` if you want to completely turn off incident creation.
#   If set to `['never']`, then this module will not create any incidents at all.
# @param [String] servicenow_credentials_validation_table
#   The table to read for validating the provided ServiceNow credentials.
#   You should set this to another table if the current set of credentials
#   don't have READ access to the default 'incident' table. Note that you
#   can turn the ServiceNow credentials validation off by setting this
# @param [Optional[Array[String[1]]]] include_facts
#   An array of fact queries to send with each event. The query can be the simple
#   name of a top level fact like 'id', or it can be a dot notation query for
#   nested facts like 'os.distro'
# @param [Enum['yaml', 'pretty_json', 'json']] facts_format
#   The format of the facts that are included in the event description
# @param [Optional[Boolean]] skip_certificate_validation
#   If your Servicenow instance uses a certificate that is not trusted by the
#   Puppet server, you can set this parameter to 'true'. The connection will
#   still use SSL, but the module will not perform certificate validation, which
#   is a risk for man in the middle attacks.
# @param [Optional[Variant[Integer[0], Float[0]]]] http_read_timeout
#   The read timeout parameter sets an upper limit on how long HTTP read
#   read operations should take. For the sake simplicity, this paramter sets
#   three different arguments to the Net::HTTP.start method: read_timeout,
#   connect_timeout, ssl_timeout. This type is a Float because those argumants
#   can all accept fractional seconds values. The default value of 60 seconds
#   is also the default value for that Ruby class if no argument is passed
# @param [Optional[Variant[Integer[0], Float[0]]]] http_write_timeout
#   Sets the write_timeout argument to the Net::HTTP.start method. The datatype
#   is a float because it accepts fractions seconds values. The default value of
#   60 seconds is also the default value for that Ruby class if no argument is
#   passed
# @param [Enum['selfsigned', 'truststore', 'none']] pe_console_cert_validation
#   Sets the method the settings validation script will use to validate the PE
#   Console certificate. The default is `selfsigned` meaning the script expects
#   the certificate generated by the PE private CA and will validate against
#   that cert. If you have generated your own cert using an alternate CA, and
#   you have imported the CA for that cert into the OS Trust Store, then use
#   `truststore`. If you would like to skip certificate validation use `none`.
class servicenow_reporting_integration::incident_management (
  String[1] $instance,
  String[1] $caller_id,
  Optional[String[1]] $user                                                                  = undef,
  Optional[Sensitive[String[1]]] $password                                                   = undef,
  Optional[Sensitive[String[1]]] $oauth_token                                                = undef,
  Optional[String[1]] $pe_console_url                                                        = undef,
  Optional[String[1]] $category                                                              = undef,
  Optional[String[1]] $subcategory                                                           = undef,
  Optional[String[1]] $contact_type                                                          = undef,
  Optional[Integer] $state                                                                   = undef,
  Optional[Integer] $impact                                                                  = undef,
  Optional[Integer] $urgency                                                                 = undef,
  Optional[String[1]] $assignment_group                                                      = undef,
  Optional[String[1]] $assigned_to                                                           = undef,
  Optional[Array[String[1]]] $allow_list                                                     = ['all'],
  Optional[Array[String[1]]] $block_list                                                     = ['none'],
  Servicenow_reporting_integration::ReportCategories $incident_creation_conditions           = ['failures', 'corrective_changes'],
  String $servicenow_credentials_validation_table                                            = 'incident',
  Optional[Array[String[1]]] $include_facts                                                  = ['aio_agent_version', 'id', 'memorysize', 'memoryfree', 'ipaddress', 'ipaddress6', 'os.distro', 'os.windows', 'path', 'uptime', 'rubyversion'],
  Enum['yaml', 'pretty_json', 'json'] $facts_format                                          = 'yaml',
  Optional[Boolean] $skip_certificate_validation                                             = false,
  Optional[Variant[Integer[0], Float[0]]] $http_read_timeout                                 = 60,
  Optional[Variant[Integer[0], Float[0]]] $http_write_timeout                                = 60,
  Enum['selfsigned', 'truststore', 'none'] $pe_console_cert_validation                       = 'selfsigned',
) {
  class { 'servicenow_reporting_integration':
    operation_mode                          => 'incident_management',
    instance                                => $instance,
    caller_id                               => $caller_id,
    user                                    => $user,
    password                                => $password,
    oauth_token                             => $oauth_token,
    pe_console_url                          => $pe_console_url,
    category                                => $category,
    subcategory                             => $subcategory,
    contact_type                            => $contact_type,
    state                                   => $state,
    impact                                  => $impact,
    urgency                                 => $urgency,
    assignment_group                        => $assignment_group,
    assigned_to                             => $assigned_to,
    allow_list                              => $allow_list,
    block_list                              => $block_list,
    incident_creation_conditions            => $incident_creation_conditions,
    servicenow_credentials_validation_table => $servicenow_credentials_validation_table,
    include_facts                           => $include_facts,
    facts_format                            => $facts_format,
    skip_certificate_validation             => $skip_certificate_validation,
    http_read_timeout                       => $http_read_timeout,
    http_write_timeout                      => $http_write_timeout,
    pe_console_cert_validation              => $pe_console_cert_validation,
  }
}
